/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <tizen.h>
#include <widget_app.h>
#include <app.h>
#include <dlog.h>
#include <app_preference.h>
#include <bundle.h>

#include "view.h"
#include "data.h"
#include "main.h"

#define POPUP_DURATION 4.0
#define INSTANCE_ID_FOR_APP_CONTROL "widget_instance_id_for_app_control"

static void _alarm_on_off_clicked_cb(void *user_data);
static void _no_alarm_down_cb(void *user_data, Evas *e, Evas_Object *obj, void *event_info);
static void _no_alarm_up_cb(void *user_data, Evas *e, Evas_Object *obj, void *event_info);
static void _no_alarm_clicked_cb(void *user_data, Evas_Object *obj, void *event_info);
static void _destroy_alarm(void *user_data);
static void _set_alarm(bundle *b, void *user_data);
static void _set_on_off(bundle *b, void *user_data);

/**
 * @brief The callback function called after widget instance is created.
 * @param[in] context The context of widget instance
 * @param[in] content The data set for the previous status
 * @param[in] w The pixel value for widget width
 * @param[in] h The pixel value for widget height
 * @param[in] user_data The user data passed from widget_app_class_create function
 */
static int widget_instance_create(widget_context_h context, bundle *content, int w, int h, void *user_data)
{
	widget_instance_data_s *wid = (widget_instance_data_s *)malloc(sizeof(*wid));
	const char *instance_id = NULL;
	char file_path[PATH_MAX] = { 0, };

	if (content != NULL) {
		/* Recover the previous status with the bundle object. */
	}

	memset(wid, 0, sizeof(*wid));

	widget_app_context_set_tag(context, wid);

	instance_id = widget_app_get_id(context);
	if (instance_id == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Failed to get instance id");
		return WIDGET_ERROR_FAULT;
	}
	wid->instance_id = instance_id;
	dlog_print(DLOG_INFO, LOG_TAG, "widget instance id : %s", instance_id);

	/*
	 * Create basic GUI
	 */
	view_create(context, w, h);
	if (wid == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to create a view.");
		return WIDGET_ERROR_FAULT;
	}

	/* Get EDJ file path will be used for GUI */
	data_get_resource_path(EDJ_PATH, file_path, sizeof(file_path));
	dlog_print(DLOG_INFO, LOG_TAG, "file path: %s", file_path);

	/*
	 * Create GUI for alarm widget
	 */
	view_alarm_widget_create(context, file_path);

	/* Set text to the given part as a second parameter */
	view_set_text(wid->layout, "no_alarm.title", "Alarm");

	/* Create and Set the button to the given part with 'focus' theme and several callback functions */
	view_set_button(wid->layout, "swallow.no_alarm.button", "focus", NULL, NULL, _no_alarm_down_cb, _no_alarm_up_cb, _no_alarm_clicked_cb, wid);
	view_set_text(wid->layout, "no_alarm.text", "Add alarm");

	return WIDGET_ERROR_NONE;
}

/**
 * @brief The callback function called before widget instance will be destroyed.
 * @param[in] context The context of widget instance
 * @param[in] reason The reason for destruction
 * @param[in,out] content The data set to save
 * @param[in] user_data The user data passed from widget_app_class_create function
 */
static int widget_instance_destroy(widget_context_h context, widget_app_destroy_type_e reason, bundle *content_info, void *user_data)
{
	void *wid = NULL;
	dlog_print(DLOG_INFO, LOG_TAG, "Widget instance destroy");

	if (reason != WIDGET_APP_DESTROY_TYPE_PERMANENT) {
		widget_app_context_set_content_info(context, content_info);
	}

	/* Get the data structure using the tag */
	widget_app_context_get_tag(context, (void**) &wid);

	/* Destroy all views */
	view_widget_destroy(wid);

	return WIDGET_ERROR_NONE;
}

/**
 * @brief The callback function called when the widget is invisible.
 * @param[in] context The context of widget instance
 * @param[in] user_data The user data passed from widget_app_class_create function
 */
static int widget_instance_pause(widget_context_h context, void *user_data)
{
	/* Take necessary actions when widget instance becomes invisible. */
	return WIDGET_ERROR_NONE;
}

/**
 * @brief The callback function called when the widget is visible.
 * @param[in] context The context of widget instance
 * @param[in] user_data The user data passed from widget_app_class_create function
 */
static int widget_instance_resume(widget_context_h context, void *user_data)
{
	/* Take necessary actions when widget instance becomes visible. */
	return WIDGET_ERROR_NONE;
}

/**
 * @brief The callback function called when the update event is occurred.
 * @param[in] context The context of widget instance
 * @param[in] content The data set for updating this widget
 * @param[in] force Although the widget is paused, if it is TRUE, the widget can be updated
 * @param[in] user_data The user data passed from widget_app_class_create function
 */
static int widget_instance_update(widget_context_h context, bundle *content, int force, void *user_data)
{
	widget_instance_data_s *wid = NULL;
	char *op = NULL;
	int ret = BUNDLE_ERROR_NONE;

	if (context == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Failed to update view : invalid parameter");
		return WIDGET_ERROR_INVALID_PARAMETER;
	}

	if (content == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Failed to get content");
		return WIDGET_ERROR_INVALID_PARAMETER;
	}

	widget_app_context_get_tag(context, (void **) &wid);
	if (wid == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Failed to get widget instance data");
		return WIDGET_ERROR_FAULT;
	}

	ret = bundle_get_str(content, "Operation", &op);
	if (ret != BUNDLE_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Failed to get bundle data : Operation");
		return WIDGET_ERROR_FAULT;
	}

	dlog_print(DLOG_DEBUG, LOG_TAG, "[%d] %s : operation (%s)", __LINE__, __func__, op);

	if (!strcmp(op, "Destroy")) {
		_destroy_alarm(wid);
	} else if (!strcmp(op, "SetAlarm")) {
		_set_alarm(content, wid);
	} else if (!strcmp(op, "SetOnOff")) {
		_set_on_off(content, wid);
	} else {
		dlog_print(DLOG_ERROR, LOG_TAG, "Failed to update widget : operation error(%s)", op);
		return WIDGET_ERROR_FAULT;
	}

	return WIDGET_ERROR_NONE;
}

/**
 * @brief The callback function called when the size of the widget is changed.
 * @param[in] context The context of widget instance
 * @param[in] w The pixel value for widget width
 * @param[in] h The pixel value for widget height
 * @param[in] user_data The user data passed from widget_app_class_create function
 */
static int widget_instance_resize(widget_context_h context, int w, int h, void *user_data)
{
	void *wid = NULL;

	widget_app_context_get_tag(context, (void**) &wid);
	view_resize(wid, w, h);

	return WIDGET_ERROR_NONE;
}

/**
 * @brief The callback function called when the language is changed.
 * @param[in] event_info Event_info is a pointer to a data structure about event
 * @param[in] user_data Data passed to the function
 */
static void widget_app_lang_changed(app_event_info_h event_info, void *user_data)
{
	/* APP_EVENT_LANGUAGE_CHANGED */
	char *locale = NULL;
	app_event_get_language(event_info, &locale);
	elm_language_set(locale);
	free(locale);
}

/**
 * @brief The callback function called when the region is changed.
 * @param[in] event_info Event_info is a pointer to a data structure about event
 * @param[in] user_data Data passed to the function
 */
static void widget_app_region_changed(app_event_info_h event_info, void *user_data)
{
	/* APP_EVENT_REGION_FORMAT_CHANGED */
}

/**
 * @brief This function handle the widget create event.
 * @param[in] user_data Data passed to the function
 */
static widget_class_h widget_app_create(void *user_data)
{
	app_event_handler_h handlers[5] = {NULL, };

	widget_app_add_event_handler(&handlers[APP_EVENT_LANGUAGE_CHANGED],
			APP_EVENT_LANGUAGE_CHANGED, widget_app_lang_changed, user_data);
	widget_app_add_event_handler(&handlers[APP_EVENT_REGION_FORMAT_CHANGED],
			APP_EVENT_REGION_FORMAT_CHANGED, widget_app_region_changed,
			user_data);

	widget_instance_lifecycle_callback_s ops = {
			.create = widget_instance_create,
			.destroy = widget_instance_destroy,
			.pause = widget_instance_pause,
			.resume = widget_instance_resume,
			.update = widget_instance_update,
			.resize = widget_instance_resize,
	};

	return widget_app_class_create(ops, user_data);
}

/**
 * @brief This function handle the widget terminate event.
 * @param[in] user_data Data passed to the function
 */
static void widget_app_terminate(void *user_data)
{
	/* Release all resources. */
}

/**
 * @brief Main function of the widget.
 */
int main(int argc, char *argv[])
{
	widget_app_lifecycle_callback_s ops = {0, };
	int ret;

	ops.create = widget_app_create;
	ops.terminate = widget_app_terminate;

	ret = widget_app_main(argc, argv, &ops, NULL);
	if (ret != WIDGET_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG, "widget_app_main() failed. err = %d", ret);

	return ret;
}

/**
 * @brief The function operated on widget click.
 * @param[in] user_data Structure has informations for managing this widget
 */
static void _alarm_on_off_clicked_cb(void *user_data)
{
	widget_instance_data_s *wid = NULL;
	Evas_Event_Flags event_flag;
	char *on_off = NULL;

	wid = user_data;
	if (wid == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "[%d] %s : invalid parameter");
		return;
	}

	/**
	 * Check whether this event is CLICK or MOUSE UP.
	 */
	event_flag = evas_event_default_flags_get(evas_object_evas_get(wid->win));

	/**
	 * Click event on scroll, we'll not deal with.
	 */
	if (event_flag == EVAS_EVENT_FLAG_ON_HOLD) {
		return;
	}

	if (wid->alarm_on) {
		on_off = "Off";
		view_send_signal_to_edje(wid->alarm_view, "off", "alarm");
		wid->alarm_on = 0;
	} else {
		on_off = "On";
		view_send_signal_to_edje(wid->alarm_view, "on", "alarm");
		wid->alarm_on = 1;
	}

	preference_set_string(wid->alarm_id, on_off);
	dlog_print(DLOG_DEBUG, LOG_TAG, "[%d] %s : key(%s), val(%s)", __LINE__, __func__, wid->alarm_id, on_off);
}

/**
 * @brief This function will be operated when the button is pressed.
 * @param[in] user_data Data will be the same value passed to evas_object_event_callback_add() as the data parameter
 * @param[in] e The e lets the user know what evas canvas the event occurred on
 * @param[in] obj Object handles on which the event occurred
 * @param[in] event_info Event_info is a pointer to a data structure about event
 */
static void _no_alarm_down_cb(void *user_data, Evas *e, Evas_Object *obj, void *event_info)
{
}

/**
 * @brief This function will be operated when the button is released.
 * @param[in] user_data Data will be the same value passed to evas_object_event_callback_add() as the data parameter
 * @param[in] e The e lets the user know what evas canvas the event occurred on
 * @param[in] obj Object handles on which the event occurred
 * @param[in] event_info Event_info is a pointer to a data structure about event
 */
static void _no_alarm_up_cb(void *user_data, Evas *e, Evas_Object *obj, void *event_info)
{
}

/**
 * @brief This function will be operated when the button is clicked.
 * @param[in] user_data Data has the same value passed to evas_object_smart_callback_add() as the data parameter
 * @param[in] obj The obj is a handle to the object on which the event occurred
 * @param[in] event_info A pointer to data which is totally dependent on the smart object's implementation and semantic for the given event
 */
static void _no_alarm_clicked_cb(void *user_data, Evas_Object *obj, void *event_info)
{
	app_control_h app_control;
	widget_instance_data_s *wid = user_data;

	/* app_control is used to launch the other application */
	app_control_create(&app_control);
	app_control_set_operation(app_control, APP_CONTROL_OPERATION_DEFAULT);

	/* 'org.example.alarm' is triggered to the app_control */
	app_control_set_app_id(app_control, "org.example.alarm");

	app_control_add_extra_data(app_control, INSTANCE_ID_FOR_APP_CONTROL, wid->instance_id);

	/* Request to launch the other application */
	if (app_control_send_launch_request(app_control, NULL, NULL) == APP_CONTROL_ERROR_NONE) {
		dlog_print(DLOG_INFO, LOG_TAG, "Succeeded to launch a alarm");
	} else {
		/* This case is for Unconnected with other application */
		widget_instance_data_s *wid = user_data;
		Evas_Object *popup = NULL;
		char *popup_text = NULL;

		dlog_print(DLOG_ERROR, LOG_TAG, "Failed to launch a alarm");

		/* Get the text which you want to show on the popup */
		popup_text = data_get_popup_text();

		/* Create Circular popup with given text and time */
		popup = view_create_text_popup(wid->layout, POPUP_DURATION, popup_text);

		/* Set the function will be called when the popup time is up */
		free(popup_text);
	}

	/* Destroy the app_control after request */
	app_control_destroy(app_control);

}

/**
 * @brief Destroys alarm.
 * @param[in] user_data Data passed to the function
 */
static void _destroy_alarm(void *user_data)
{
	widget_instance_data_s *wid = NULL;

	wid = user_data;
	if (wid == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Invalid parameter");
		return;
	}

	dlog_print(DLOG_INFO, LOG_TAG, "Alarm application is terminated");

	/* Check current view is alarm_view */
	Elm_Object_Item *top_item = NULL;
	top_item = elm_naviframe_top_item_get(wid->nf);

	if (wid->basic_item != top_item) {
		dlog_print(DLOG_INFO, LOG_TAG, "Delete alarm_view");
		elm_naviframe_item_pop(wid->nf);

		wid->alarm_view = 0;
		wid->alarm_on = 1;
	}
}

/**
 * @brief Sets alarm.
 * @param[in] user_data Data passed to the function
 */
static void _set_alarm(bundle *b, void *user_data)
{
	widget_instance_data_s *wid = NULL;
	char file_path[PATH_MAX] = { 0, };
	char *alarm_time = NULL;
	char *on_off = NULL;
	char *alarm_id = NULL;
	int ret = BUNDLE_ERROR_NONE;

	wid = user_data;
	if (wid == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Failed to set alarm : invalid parameter");
		return;
	}

	if (b == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Failed to set alarm : bundle is NULL");
		return;
	}

	ret = bundle_get_str(b, "AlarmTime", &alarm_time);
	if (ret != BUNDLE_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Failed to get bundle data : AlarmTime");
		return;
	}

	ret = bundle_get_str(b, "OnOff", &on_off);
	if (ret != BUNDLE_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Failed to get bundle data : OnOff");
		return;
	}

	ret = bundle_get_str(b, "AlarmId", &alarm_id);
	if (ret != BUNDLE_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Failed to get bundle data : AlarmId");
		return;
	}

	dlog_print(DLOG_DEBUG, LOG_TAG, "[%d] %s : AlarmTime(%s), OnOff(%s), AlarmId(%s)", __LINE__, __func__, alarm_time, on_off, alarm_id);

	wid->alarm_id = strdup(alarm_id);

	/* Get the file path of EDJ */
	data_get_resource_path(EDJ_PATH, file_path, sizeof(file_path));

	/* Create alarm widget view with 'alarm.view' group in EDJ file */
	wid->alarm_view = view_create_layout(wid->nf, file_path, "alarm.view", NULL, NULL);
	if (wid->alarm_view == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to create a layout of base alarm.");
		return ;
	}

	/* Set the text 'alarm_time' to the 'widget.time' part */
	view_set_text(wid->alarm_view, "widget.time", alarm_time);

	/* Push alarm view to naviframe to manage several view easily */
	view_push_item_to_naviframe(wid->nf, wid->alarm_view, NULL, NULL);

	wid->alarm_on = 1;

	/* Set the function will be called when the widget is clicked */
	view_set_event_callback(wid->alarm_view, "widget.event", WIDGET_EVENT_CLICKED, _alarm_on_off_clicked_cb, wid);

}

/**
 * @brief Sets on/off state of alarm.
 * @param[in] b Bundle data passed by UI application
 * @param[in] user_data Data passed to the function
 */
static void _set_on_off(bundle *b, void *user_data)
{
	widget_instance_data_s *wid = NULL;
	char *on_off = NULL;
	int ret = BUNDLE_ERROR_NONE;

	wid = user_data;
	if (wid == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Failed to set alarm on/off : invalid parameter");
		return;
	}

	if (b == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Failed to set alarm on/off : bundle is NULL");
		return;
	}

	ret = bundle_get_str(b, "OnOff", &on_off);
	if (ret != BUNDLE_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Failed to get bundle data : OnOff");
		return;
	}

	if (!strcmp(on_off, "On")) {
		if (wid->alarm_on == 0) {
			view_send_signal_to_edje(wid->alarm_view, "on", "alarm");
			wid->alarm_on = 1;
		}
	} else {
		if (wid->alarm_on == 1) {
			view_send_signal_to_edje(wid->alarm_view, "off", "alarm");
			wid->alarm_on = 0;
		}
	}
}
/* End of the file */
